/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.yjs;



import casa.dodwan.run.Dodwan;
import casa.dodwan.run.dodwand;
import casa.util.Logger;

import casa.dodwan.util.SystemEnvironment;

import casa.dodwan.util.Environment;
import casa.dodwan.util.Sink;


import casa.dodwan.message.Message;
import casa.dodwan.docware.Descriptor;
import casa.dodwan.napi.DodwanPdu;
import casa.dodwan.napi.ws.DodwanWsEndpoint;
import casa.dodwan.napi.ws.DodwanWsServer;
/**
 *
 * 
 * Service for processing incoming Yjs messages.
 * The service simply relays YJS received messages via NAPI to the destinee that is specified in the descriptor
 * 
 * When receiving a message from another peer (as a sink registered in the DoDWAN transmission service).
 * it sends this message as a NAPI notification through the websocket
 * 
 */
public class YjsService  implements Sink<Message> {
    
    private Dodwan dodwan = null;
    private String host = SystemEnvironment.host;
    
    
    public YjsService(dodwand dodwand) {
        this.dodwan = dodwand.dodwan;
    }
    
    
    // ------------------------------------------------------------
    public void startService() {
        this.dodwan.transmissionService.dispatcher.addSink(this);
        Logger.log("yjs", "Starting the YjsService");      
    }
    
    
    // ------------------------------------------------------------
    public void stopService() {
	Logger.log("yjs", "Stopping YjsService");
	this.dodwan.transmissionService.dispatcher.removeSink(this);
    }
    
    
    // ------------------------------------------------------------
    /*
    *  Called by the DoDWAN transmission service when a message arrives
    *  (implementation of the Sink interface)
    *  Transfers the descriptor of the message (no payload) in a PDU 
    *  through the websocket
    */
    @Override
     public synchronized void write (Message msg) throws Exception {

     	Descriptor desc = msg.getDescriptor();
        
        //System.err.println("YjsService got a message : "+desc);
        
         
     	String src = desc.getAttribute("_sdr");
        //System.err.println("YjsService: src="+src);
        
     	if (src.equals(this.host)) {
     	    // Ignoring messages that have been sent locally (should never happen)
     	    
     	    return;
     	}

     	String typeYjs = desc.getAttribute("_type");
        //System.err.println("YjsService: _type="+typeYjs);
     	// Sends a pdu only for YJS messages
     	if (typeYjs.equals("yjs"))  {   
           // Sends the message as a notification (descriptor) on the websocket  
           DodwanPdu notif = new DodwanPdu();
           notif.putString("name", "recv_from_peer");
           String mid = desc.getDocumentId();
           notif.putString("mid", mid);
           notif.putAttributes("desc", desc.asAttributes());
           //System.err.println("YjsService sending notif PDU "+ notif);
           
           // We assume that there is only one DodwanWsEndpoint
           DodwanWsServer.getEndpoints().get(0).sendPdu(notif);
        }
     }
     
    
     
       
}
